use strict;
use Readonly;
use Test::More;
use Test::Exception;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::MultiStoreProvider::API::ShopRef qw (IsSubShopAssignedToMasterShop AssignSubShopToMasterShop UnassignSubShopFromMasterShop );

use constant SUBSHOPCOUNT => 3;

sub Main {
    SKIP: {
            skip 'Not running on site' unless GetTestSite() eq 'Site';

            # Load default provider "distributor".
            my $Provider = LoadRootObject()->child('Providers')->child('Distributor');

            # Startup and init.
            ok( Init($Provider), 'initialization' );

            # Load ShopRefs of provider.
            my $ShopRefs = $Provider->child('ShopRefs');

            # Run tests with eval, to avoid hard errors.
            eval {
                TestAssignSubShop($ShopRefs);
                TestListSubShops($ShopRefs);
            };

            # Error case.
            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            # Final cleanup.
            ok( Cleanup($ShopRefs), 'cleanup' );

    }
    done_testing();
}

sub TestAssignSubShop {
    my ($ShopRefs) = @_;

    # Get masterref and first testsubshopref.
    my $MasterShopRef = $ShopRefs->child('TestMasterShop');
    my $SubShopRef    =  $ShopRefs->child('TestSubShop1');

    # Check if not assigned.
    ok( !IsSubShopAssignedToMasterShop( $MasterShopRef, $SubShopRef ), 'subshop not assigned' );
    # Assign.
    AssignSubShopToMasterShop( $MasterShopRef, $SubShopRef );
    # Check if assigned.
    ok( IsSubShopAssignedToMasterShop( $MasterShopRef, $SubShopRef ), 'subshop is assigned' );
    # Check if error is thrown when double assignment.
    dies_ok( sub { AssignSubShopToMasterShop( $MasterShopRef, $SubShopRef ) }, 'duplicate assignment dies' );
    # Unassign.
    UnassignSubShopFromMasterShop($SubShopRef);
    # Check if unassigned.
    ok( !IsSubShopAssignedToMasterShop( $MasterShopRef, $SubShopRef ), 'subshop again not assigned' );
}

sub TestListSubShops {
    my ($ShopRefs) = @_;

    # Get master shopref.
    my $MasterShopRef =  $ShopRefs->child('TestMasterShop');

    # Run tests on each testsubshopref.
    for (my $i = 1; $i <= SUBSHOPCOUNT; $i++) {
        # Load test subshop.
        my $SubShopRef =  $ShopRefs->child('TestSubShop' . $i);
        # Assign.
        AssignSubShopToMasterShop( $MasterShopRef, $SubShopRef );
        # Check if assigned.
        is( $SubShopRef->get('MasterShopRef')->id, $MasterShopRef->id, 'mastershopref id matches' );
    }
    # Check if all testsubshops assigned mastershopref.
    is( scalar @{ $MasterShopRef->get('SubShopRefs') }, SUBSHOPCOUNT, SUBSHOPCOUNT . ' assigned subshops' );
}

sub Cleanup {
    my ($ShopRefs) = @_;

    # Skip cleanup if SKIP_CLEANUP is set.
    return 1 if $ENV{'SKIP_CLEANUP'};

    # Delete testmastershopref.
    $ShopRefs->child('TestMasterShop')->delete() if $ShopRefs->existsChild('TestMasterShop');

    # Delete testsubshoprefs.
    for (my $i = 1; $i <= SUBSHOPCOUNT; $i++) {
         $ShopRefs->child('TestSubShop' . $i)->delete() if $ShopRefs->existsChild('TestSubShop' . $i);
    }

    return 1;
}

sub Init {
    my ($Provider) = @_;

    # Skip init if SKIP_CLEANUP is set.
    return 1 if $ENV{'SKIP_CLEANUP'};

    # Make initial cleanup.
    Cleanup($Provider->child('ShopRefs'));

    # Get Siteobject.
    my $System = LoadRootObject();

    # Build baseurl for webservices.
    my $ProtocolAndServer = $System->get('ProtocolAndServer');
    my $WebServiceBaseURL = $ProtocolAndServer . '/epages/';

    # Get required stores.
    my $Stores         = $System->child('Stores');
    my $Master         = $Stores->child('Master');
    my $MultiStore     = $Stores->child('MultiStore');

    # Get required shoptypes.
    my $ShopTypes = $Provider->child('ShopTypes');
    my $MasterShopType = $ShopTypes->child('Demo');
    my $SubShopType = $ShopTypes->child('ERetail1000');

    # Get or create testmaster.
    my $MasterShopRef = $Provider->insertShop({
        'Alias' => 'TestMasterShop',
        'Store' => $Master,
        'ShopAlias' => 'TestMasterShop',
        'WebServerScriptNamePart' => 'TestMasterShop',
        'ShopType' => $MasterShopType,
    });

    # Setup webserivce for master.
    my $hValues = {
        'MS_WebServiceLogging' => {
            'TestMasterShop' => 1,
            'Site' => 1,
        },
        'MS_WebServiceURL' => {
           'Site' => $WebServiceBaseURL . 'Site.soap',
           'TestMasterShop' => $WebServiceBaseURL. 'TestMasterShop.soap',
        },
        'MS_WebServiceUser' => {
           'Site' => '/Users/wsadmin',
           'TestMasterShop' => '/Users/wsadmin',
        },
        'MS_WebServicePassword' => {
           'Site' => 'wsadmin',
           'TestMasterShop' => 'wsadmin',
        },
    };
    $MasterShopRef->remoteShop->set( $hValues );

    # Create test subshops.
    for (my $i = 1; $i <= SUBSHOPCOUNT; $i++) {
        my $SubShopAlias = 'TestSubShop' . $i;
        my $SubShopRef = $Provider->insertShop({
            'Alias' => $SubShopAlias,
            'Store' => $MultiStore,
            'ShopAlias' => $SubShopAlias,
            'WebServerScriptNamePart' => $SubShopAlias,
            'ShopType' => $SubShopType,
        });

        # Setup webservice for subshops.
        $hValues = {
            'MS_WebServiceLogging' => {
                $SubShopAlias => 1,
            },
            'MS_WebServiceURL' => {
                $SubShopAlias => $WebServiceBaseURL . $SubShopAlias . '.soap',
            },
            'MS_WebServiceUser' => {
                $SubShopAlias => '/Users/wsadmin',
            },
            'MS_WebServicePassword' => {
                $SubShopAlias => 'wsadmin',
            },
        };
        $SubShopRef->remoteShop->set( $hValues );

    }

    return 1;
}

RunScript( Sub => sub {RunOnStore('Store' => GetTestSite(),'Sub' => \&Main , Debug=>1);} );

1;
